@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: mcmd.c 968 2005-10-11 22:34:22Z grondo $
+ *  $Id: mcmd.c 1009 2006-02-14 16:11:43Z grondo $
  *****************************************************************************
- *  Copyright (C) 2001-2002 The Regents of the University of California.
+ *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Jim Garlick <garlick@llnl.gov>.
  *  UCRL-CODE-2003-005.
@@ -205,10 +205,10 @@ mcmd_init(opt_t * opt)
     int rv, rand_fd;
 
     /*
-     * Drop elevated permissions if we have them.
+     * Drop privileges if running setuid root
      */
-    setuid(getuid());
-    setgid(getgid());
+    if ((geteuid() == 0) && (getuid() != 0))
+        setuid (getuid ());
 
     /*
      * Generate a random number to send in our package to the 
@@ -288,7 +288,7 @@ encode_localhost_string (const char *host, char *str, int maxlen)
  * Connection can time out.
  *      ahost (IN)              target hostname
  *      addr (IN)               4 byte internet address
- *      locuser (IN)            not used 
+ *      locuser (IN)            local username
  *      remuser (IN)            remote username
  *      cmd (IN)                remote command to execute under shell
  *      rank (IN)               not used 
