@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: dsh.c 965 2005-10-10 22:12:29Z grondo $
+ *  $Id: dsh.c 1035 2006-03-22 20:36:00Z grondo $
  *****************************************************************************
- *  Copyright (C) 2001-2002 The Regents of the University of California.
+ *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Jim Garlick <garlick@llnl.gov>.
  *  UCRL-CODE-2003-005.
@@ -111,7 +111,7 @@
 #include "dsh.h"
 #include "opt.h"
 #include "wcoll.h"
-#include "mod_rcmd.h"
+#include "rcmd.h"
 
 static int debug = 0;
 
@@ -239,7 +239,7 @@ static void _fwd_signal(int signum)
 
     for (i = 0; t[i].host != NULL; i++) {
         if ((t[i].state == DSH_READING)) 
-			mod_rcmd_signal(t[i].rcmd, signum);
+			rcmd_signal(t[i].rcmd, signum);
     }
 
 }
@@ -596,16 +596,16 @@ static void *_rcp_thread(void *args)
     _int_block();               /* block SIGINT */
 
 #if	HAVE_MTSAFE_GETHOSTBYNAME
-    if (a->resolve_hosts)
+    if (a->rcmd->opts->resolve_hosts)
         _gethost(a->host, a->addr);
 #endif
     a->start = time(NULL);
     a->state = DSH_RCMD;
 
-    a->rcmd = mod_rcmd_create (a->host, a->addr, a->luser, a->ruser, 
-                               a->cmd, a->nodeid, a->dsh_sopt);
+    rcmd_connect (a->rcmd, a->host, a->addr, a->luser, a->ruser, 
+                  a->cmd, a->nodeid, a->dsh_sopt);
 
-    if (a->rcmd == NULL || a->rcmd->fd == -1)
+    if (a->rcmd->fd == -1)
         result = DSH_FAILED;
     else {
         a->connect = time(NULL);
@@ -648,7 +648,7 @@ static void *_rcp_thread(void *args)
     a->state = result;
     a->finish = time(NULL);
 
-    rc = mod_rcmd_destroy (a->rcmd);
+    rc = rcmd_destroy (a->rcmd);
     if ((a->rc == 0) && (rc > 0))
         a->rc = rc;
 
@@ -760,7 +760,7 @@ static void *_rsh_thread(void *args)
     a->start = time(NULL);
 
 #if	HAVE_MTSAFE_GETHOSTBYNAME
-    if (a->resolve_hosts)
+    if (a->rcmd->opts->resolve_hosts)
         _gethost(a->host, a->addr);
 #endif
     _xsignal (SIGPIPE, SIG_BLOCK);
@@ -768,14 +768,10 @@ static void *_rsh_thread(void *args)
     /* establish the connection */
     a->state = DSH_RCMD;
 
-    a->rcmd = mod_rcmd_create(a->host, a->addr, a->luser, a->ruser,
-                              a->cmd, a->nodeid, a->dsh_sopt);
+    rcmd_connect (a->rcmd, a->host, a->addr, a->luser, a->ruser,
+                  a->cmd, a->nodeid, a->dsh_sopt);
 
-    /* 
-     * Copy stdout/stderr to local stdout/stderr, 
-     * appropriately tagged.
-     */
-    if (a->rcmd == NULL || a->rcmd->fd == -1) {
+    if (a->rcmd->fd == -1) {
         result = DSH_FAILED;    /* connect failed */
     } else {
         /* connected: update status for watchdog thread */
@@ -812,7 +808,7 @@ static void *_rsh_thread(void *args)
                 else
                     err("%p: %S: xpoll: %m\n", a->host);
                 result = DSH_FAILED;
-                mod_rcmd_signal (a->rcmd, SIGTERM);
+                rcmd_signal (a->rcmd, SIGTERM);
                 break;
             }
 
@@ -848,7 +844,7 @@ static void *_rsh_thread(void *args)
     _flush_output (a->outbuf, (out_f) out, a);
     _flush_output (a->errbuf, (out_f) err, a);
 
-    rv = mod_rcmd_destroy (a->rcmd);
+    rv = rcmd_destroy (a->rcmd);
     if ((a->rc == 0) && (rv > 0))
         a->rc = rv;
 
@@ -959,7 +955,6 @@ static int _thd_init (thd_t *th, opt_t *opt, List pcp_infiles, int i)
     th->ruser = opt->ruser;
     th->state = DSH_NEW;
     th->labels = opt->labels;
-    th->rcmd = NULL;
     th->nodeid = i;
     th->cmd = opt->cmd;
     th->dsh_sopt = opt->separate_stderr;  /* dsh-specific */
@@ -972,10 +967,13 @@ static int _thd_init (thd_t *th, opt_t *opt, List pcp_infiles, int i)
     th->kill_on_fail = opt->kill_on_fail;
     th->outbuf = cbuf_create (64, 8192);
     th->errbuf = cbuf_create (64, 8192);
+
+    th->rcmd = rcmd_create (th->host);
+
 #if	!HAVE_MTSAFE_GETHOSTBYNAME
     /* if MT-safe, do it in parallel in rsh/rcp threads */
     /* gethostbyname_r is not very portable so skip it */
-    if (opt->resolve_hosts)
+    if (th->rcmd->opts->resolve_hosts)
         _gethost(th->host, th->addr);
 #endif
 
@@ -1001,7 +999,7 @@ int dsh(opt_t * opt)
     /*
      *   Initialize rcmd modules...
      */
-    if (mod_rcmd_init(opt) < 0) {
+    if (rcmd_init(opt) < 0) {
         err("%p: unable to initialize an rcmd module\n");
 		exit(1);
     }
