@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: mod.c 965 2005-10-10 22:12:29Z grondo $
+ *  $Id: mod.c 1024 2006-03-16 00:07:45Z grondo $
  *****************************************************************************
- *  Copyright (C) 2001-2002 The Regents of the University of California.
+ *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Jim Garlick <garlick@llnl.gov>.
  *  UCRL-CODE-2003-005.
@@ -642,7 +642,10 @@ _mod_load_dynamic(const char *fq_path)
     mod->filename = Strdup(info->filename);
 
     if (_is_loaded(mod->filename)) {
-        err("%p: [%s] module already loaded\n", mod->filename);
+        /* Module already loaded. This is OK, no need for
+         *   error message. (Could have already opened a .la and
+         *   we are now opening the corresponding .so 
+         */
         goto fail;
     }
   
@@ -720,13 +723,6 @@ _mod_load_dynamic_modules(const char *dir, opt_t *pdsh_opts)
 
         strcpy(p, entry->d_name);
 
-        /*
-         *  As an efficiency enhancement, only attempt to open 
-         *    libtool ".la" files.
-         */
-        if (strcmp(&p[strlen(p) - 3], ".la") != 0)
-            continue;
-
         if (stat(path, &st) < 0)
             continue; 
         if (!S_ISREG(st.st_mode))
