@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: opt.c 933 2005-08-08 20:18:24Z grondo $
+ *  $Id: opt.c 1036 2006-03-22 20:39:20Z grondo $
  *****************************************************************************
- *  Copyright (C) 2001-2002 The Regents of the University of California.
+ *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Jim Garlick <garlick@llnl.gov>.
  *  UCRL-CODE-2003-005.
@@ -46,17 +46,20 @@
 #include "src/common/hostlist.h"
 #include "src/common/err.h"
 #include "src/common/list.h"
+#include "src/common/split.h"
 #include "src/common/xstring.h"
 #include "src/common/xmalloc.h"
 #include "dsh.h"                
 #include "opt.h"
 #include "wcoll.h"
 #include "mod.h"
-#include "mod_rcmd.h"
+#include "rcmd.h"
 
 static void _usage(opt_t * opt);
 static void _show_version(void);
 
+static char *exclude_buf = NULL;
+
 #define OPT_USAGE_DSH "\
 Usage: pdsh [-options] command ...\n\
 -S                return largest of remote command return values\n"
@@ -363,6 +366,8 @@ void opt_env(opt_t * opt)
     }
 }
 
+static void wcoll_append (opt_t *opt, char *hosts);
+
 /*
  * Override  default/environment options with command line arguments.
  *	opt (IN/OUT)	option struct	
@@ -374,7 +379,6 @@ void opt_args(opt_t * opt, int argc, char *argv[])
     int c;
     extern int optind;
     extern char *optarg;
-    char *exclude_buf = NULL;
 
 #ifdef __linux
     /* Tell glibc getopt to stop eating after the first non-option arg */
@@ -402,7 +406,7 @@ void opt_args(opt_t * opt, int argc, char *argv[])
             if (strcmp(optarg, "-") == 0)
                 opt->wcoll = read_wcoll(NULL, stdin);
             else
-                opt->wcoll = hostlist_create(optarg);
+               wcoll_append(opt, optarg);
             break;
         case 'x':              /* exclude node list */
             exclude_buf = Strdup(optarg);
@@ -473,9 +477,11 @@ void opt_args(opt_t * opt, int argc, char *argv[])
     }
 
     /*
-     *  Load requested (or default) rcmd module 
+     *  Load default module for all hosts (unless overridden)
      */
-    if (mod_rcmd_load(opt) < 0)
+    if (opt->rcmd_name == NULL)
+        opt->rcmd_name = Strdup(rcmd_get_default_module ());
+    if (rcmd_register_default_rcmd(opt->rcmd_name) < 0)
         exit(1);
 
     /* DSH: build command */
@@ -504,21 +510,6 @@ void opt_args(opt_t * opt, int argc, char *argv[])
          */
         if (mod_read_wcoll(opt) < 0)
             exit(1);
-
-        /* handle -x option */
-        if (exclude_buf != NULL && opt->wcoll) {
-            /*
-             * Delete any hosts in exclude_buf from the wcoll,  
-             *  ignoring errors (except for an invalid hostlist
-             *  in exclude_buf)
-             */
-            if (hostlist_delete(opt->wcoll, exclude_buf) <= 0) {
-                if (errno == EINVAL)
-                    errx ("%p: Invalid argument to -x \"%s\"\n", exclude_buf);
-            }
-
-            Free((void **) &exclude_buf);
-        }
     }
 }
 
@@ -536,6 +527,21 @@ bool opt_verify(opt_t * opt)
     if (mod_postop(opt) > 0)
         verified = false;
 
+    /* handle -x option */
+    if (!opt->pcp_server && exclude_buf && opt->wcoll) {
+        /*
+         * Delete any hosts in exclude_buf from the wcoll,  
+         *  ignoring errors (except for an invalid hostlist
+         *  in exclude_buf)
+         */
+        if (hostlist_delete(opt->wcoll, exclude_buf) <= 0) {
+            if (errno == EINVAL)
+                errx ("%p: Invalid argument to -x \"%s\"\n", exclude_buf);
+        }
+
+        Free((void **) &exclude_buf);
+    }
+
     /* can't prompt for command if stdin was used for wcoll */
     if (personality == DSH && opt->stdin_unavailable && !opt->cmd) {
         _usage(opt);
@@ -623,31 +629,6 @@ bool opt_verify(opt_t * opt)
 #define ALLOCSTR(x)	(x == ALLOC_BLOCK ? "ALLOC_BLOCK" : \
 			  (x == ALLOC_CYCLIC ? "ALLOC_CYCLIC" : "<Unknown>"))
 
-static char * _list_join(const char *sep, List l)
-{
-    char *str = NULL;
-    char *result = NULL;
-    ListIterator i;
-        
-    if (list_count(l) == 0)
-        return NULL;
-        
-    i = list_iterator_create(l);
-    while ((str = list_next(i))) {
-        char buf[1024];
-        snprintf(buf, 1024, "%s%s", str, sep); 
-        xstrcat(&result, buf);
-    }
-    list_iterator_destroy(i);
-
-    /* 
-     * Delete final separator
-     */
-    result[strlen(result) - strlen(sep)] = '\0';
-
-    return result;
-}
-
 /*
  * List the current options.
  *	opt (IN)	option list
@@ -667,7 +648,7 @@ void opt_list(opt_t * opt)
         out("Command:		%s\n", STRORNULL(opt->cmd));
     } else {
         out("-- PCP-specific options --\n");
-        infile_str = _list_join(", ", opt->infile_names);
+        infile_str = list_join(", ", opt->infile_names);
         if (infile_str) {
             out("Infile(s)		%s\n", infile_str);
             Free((void **) &infile_str);
@@ -734,7 +715,7 @@ void opt_free(opt_t * opt)
     if (opt->infile_names)
         list_destroy(opt->infile_names);
 
-    mod_rcmd_exit();
+    rcmd_exit();
 }
 
 /*
@@ -750,7 +731,7 @@ static char * _module_list_string(char *type)
         return NULL;
 
     l = mod_get_module_names(type);
-    names = _list_join(",", l);
+    names = list_join(",", l);
     list_destroy(l);
 
     return names;
@@ -766,7 +747,7 @@ static char *_rcmd_module_list(char *buf, int maxlen)
         goto done;
 
     if (mod_count("rcmd") > 1) {
-        char *def = mod_rcmd_get_default_module();
+        char *def = rcmd_get_default_module();
         len2 = snprintf ( buf+len, maxlen-len, " (default: %s)",
                           def ? def : "none" );
         if (len2 < 0)
@@ -825,6 +806,74 @@ static void _show_version(void)
     exit(0);
 }
 
+/*
+ *  Take a string `hosts' possibly of form "rcmd_type:user@hostlist" and
+ *    place the hostlist part of the string in *hptr and the rcmd part
+ *    of the string in rptr. Returns nonzero if an rcmd_type was found,
+ *    zero otherwise (in which case rptr is not touched).
+ */
+static int get_host_rcmd_type (char *hosts, char **rptr, char **hptr, 
+                               char **uptr)
+{
+    char *p = hosts;
+    char *q;
+    *uptr = NULL;
+    *rptr = NULL;
+    *hptr = hosts;
+
+    p = strchr (hosts, ':');
+    q = strchr (hosts, '@');
+
+    if (p && q && p > q)
+        errx ("Host spec \"%s\" not of form [rcmd_type:][user@]hosts\n", hosts);
+
+    if (p) {
+        *rptr = *hptr;
+        *p++ = '\0';
+        *hptr = p;
+    }
+
+    if (q) {
+        *uptr = *hptr;
+        *q++ = '\0';
+        *hptr = q;
+    }
+
+    return (1);
+    
+}
+
+
+static void wcoll_append (opt_t *opt, char *str)
+{
+    hostlist_t  hl = hostlist_create (str);
+    char *rcmd_type = NULL;
+    char *tok;
+
+    if (!hl)
+        return;
+
+    if (!opt->wcoll)
+        opt->wcoll = hostlist_create (NULL);
+
+    while ((tok = hostlist_shift (hl))) {
+        char *hosts, *user;
+
+        get_host_rcmd_type (tok, &rcmd_type, &hosts, &user);
+
+        hostlist_push (opt->wcoll, hosts);
+
+        if (rcmd_type || user) {
+            if (rcmd_register_defaults (hosts, rcmd_type, user) < 0)
+                errx ("Failed to register rcmd module \"%s\" for hosts \"%s\"\n",
+                      rcmd_type, hosts);
+        }
+    }
+
+    hostlist_destroy (hl);
+
+    return;
+}
 
 /*
  * vi:tabstop=4 shiftwidth=4 expandtab
